from black_mamba.deploy import DeployContract


deploy_contract_instance = DeployContract()
parameters = []
tx_params = {{ "from": "" }}
private_key = None

deploy_contract_instance.deploy_contract("{smart_contract_name}", parameters, tx_params, private_key)



from pathlib import Path


class TestWithContracts():

    tmp_path = Path('/tmp') # TODO: Change this into dynamic temp directory

    def setup_method(self, method):
        self.fixtures_dir = Path('.') / Path('test') / Path('fixtures')
        self.build_dir = self.tmp_path / Path('build')
        self.migrations_dir = self.tmp_path / Path("migrations")
        self.decentralized_app_dir = self.tmp_path / Path("decentralized_app")
        self.build_contracts_dir = self.tmp_path / Path('build') / Path('contracts')
        self.json_compiled_fixture = self.fixtures_dir / Path("HelloWorld.json")
        self.json_compiled_file = self.build_contracts_dir / Path('HelloWorld.json')
        self.migration_file = self.migrations_dir / Path("deploy_HelloWorld.py")
        self.migration2_file = self.migrations_dir / Path("deploy_HelloParameters.py")
        self.json2_compiled_fixture = self.fixtures_dir / Path("HelloParameters.json")
        self.json2_compiled_file = self.build_contracts_dir / Path('HelloParameters.json')

        if not self.build_dir.exists():
            self.build_dir.mkdir()
        if not self.build_contracts_dir.exists():
            self.build_contracts_dir.mkdir()
        if not self.migrations_dir.exists():
            self.migrations_dir.mkdir()
        if not self.decentralized_app_dir.exists():
            self.decentralized_app_dir.mkdir()

    def teardown_method(self, method):
        if self.json_compiled_file.exists():
            self.json_compiled_file.unlink()
        if self.json2_compiled_file.exists():
            self.json2_compiled_file.unlink()
        if self.migration_file.exists():
            self.migration_file.unlink()
        if self.migration2_file.exists():
            self.migration2_file.unlink()
        if self.build_contracts_dir.exists():
            self.build_contracts_dir.rmdir()
        if self.build_dir.exists():
            self.build_dir.rmdir()
        if self.migrations_dir.exists():
            self.migrations_dir.rmdir()
        if self.decentralized_app_dir.exists():
            self.decentralized_app_dir.rmdir()



from json import load
from pathlib import Path

import pytest
from web3 import (
    EthereumTesterProvider,
    Web3,
)


@pytest.fixture
def eth_tester():
    return EthereumTesterProvider().ethereum_tester

def contract(smart_contract_name, parameters=[], contract_directory=Path('.')):
    build_contracts_dir = contract_directory / Path('build') / Path('contracts')
    contract_json_file = (build_contracts_dir / smart_contract_name).with_suffix('.json')

    with open(contract_json_file, 'r') as smart_contract_build_file:
        json_object = load(smart_contract_build_file)
        bytecode = json_object["bytecode"]
        abi = json_object["abi"]
        w3 = Web3(EthereumTesterProvider())
        contract = w3.eth.contract(abi=abi, bytecode=bytecode)
        tx_hash = contract.constructor(*parameters).transact()
        tx_receipt = w3.eth.waitForTransactionReceipt(tx_hash)
        contract = w3.eth.contract(abi=abi, address=tx_receipt.contractAddress)
        return contract



networks = {
    "development": {
        "mode": "HTTP",
        "host": "localhost",
        "port": 7545,
        "network_id": "*"
    }
    #"development": {
    #    "mode": "IPC",
    #    "url": "/home/sarahconnor/ethereum/data/geth.ipc"
    #}
}



from argparse import ArgumentParser
from pathlib import Path

from black_mamba.initialization.init import initialize_project_directory
from black_mamba.compilation.vyper_compiler import compile_all_files
from black_mamba.server.geth_dev_server import run_server


def parse_cli_args():

    parser = ArgumentParser(description="Mamba Framework")
    parser.add_argument('mode', type=str, help="mode of Mamba tool")
    parser.add_argument('--httpserver', action="store_true", required=False)

    arguments = parser.parse_args()

    mode = arguments.mode
    if mode == 'init':
        initialize_project_directory(Path('.'))
    elif mode == 'compile':
        compile_all_files(Path('contracts'), Path('build') / Path('contracts'), Path("migrations"))
    elif mode == "server":
        run_server(Path("datadir"), arguments.httpserver)


if __name__ == "__main__":
    parse_cli_args()



from subprocess import Popen, PIPE
from pathlib import Path
from shutil import copyfile


def setup_datadir(datadir: Path):
    if not datadir.exists():
        datadir.mkdir()

    genesis_sample = Path(__file__).parent / Path("server_files") / Path("genesis.json")
    genesis_file = datadir / Path("genesis.json")
    copyfile(genesis_sample, genesis_file)

    private_key_sample = Path(__file__).parent / Path("server_files") / Path("private_key.json")
    private_key_file = datadir / Path("private_key.json")
    copyfile(private_key_sample, private_key_file)

def check_geth_process():
    try:
        geth_process = Popen(["geth", "version"], stdout=PIPE, stderr=PIPE)
    except FileNotFoundError:
        return False
    stdout, _ = geth_process.communicate()
    if stdout.split(b"\n")[0] == b"Geth":
        return True
    return False

def check_datadir_inited(datadir: Path):
    geth_folder = datadir / Path("geth")
    keystore_folder = datadir / Path("keystore")
    if geth_folder.exists() and keystore_folder.exists():
        return True
    return False

def init_geth_datadir(datadir: Path):
    geth_process = Popen(["geth", "--datadir", ".", "init", "genesis.json"], cwd=datadir)
    geth_process.communicate()

def run_dev_server(datadir: Path, httpserver: bool):
    geth_command = ["geth", "--networkid", "15", "--datadir", ".", "--mine", "--minerthreads", "1", "--etherbase", "0x3333333333333333333333333333333333333333"]
    if httpserver:
        geth_command.extend(["--ws", "--rpc"])
    geth_process = Popen(geth_command, cwd=datadir)
    geth_process.communicate()

def run_server(datadir: Path, httpserver: bool):
    setup_datadir(datadir)
    if not check_geth_process():
        print("You need to install Geth and put it in PATH.")
        print("Geth can be downloaded from here: https://geth.ethereum.org/downloads")
        return
    if not check_datadir_inited(datadir):
        init_geth_datadir(datadir)
    run_dev_server(datadir, httpserver)
